# La empresa Jacuzzis Rivera (JR) fabrica tres productos:
# Tina Básica (X1), Tina Chevere (X2) y Tina De Lujo (X3).
# Para el siguiente mes tiene disponibles cantidades
# limitadas de los recursos que usa (bombas de agua,
# horas de mano de obra y pies de tubería). Deseamos
# fabricar la cantidad de unidades de cada producto
# que maximice la utilidad total de la empresa. 

# MAX Z = 350 X1 + 300 X2 + 320 X3
# Subject to:
#  1 X1 +  1 X2 +  1 X3 ≤ 200       Bombas
#  9 X1 +  6 X2 +  8 X3 ≤ 1,566     Horas MdO
# 12 X1 + 16 X2 + 13 X3 ≤ 2,880     Tubería

using JuMP
using GLPK
using MathOptInterface
using DelimitedFiles

inicio = time()

# Nombres de los recursos
recursos = ["Bombas  ", "HorasMdO", "Tuberías"]

# Leemos los datos de utilidades de los productos
ut_prod = readdlm("Utilidades.txt")

# Leemos la disponibilidad de los recursos
disp_rec = readdlm("Disponibles.txt")

# Leemos los datos del consumo de recursos
# para hacer cada producto
c_rec_prod = readdlm("Consumos.txt")

# Determinamos el número de productos y de recursos
prod = size(ut_prod)[1]
rec = size(disp_rec)[1]

# Creamos el modelo
m = Model(with_optimizer(GLPK.Optimizer))

# Creamos las variables
@variable(m, X[1:prod] >= 0)

# Creamos la función objetivo
@objective(m, Max, sum(X[i]*ut_prod[i] for i in 1:prod))

# Agregamos las restricciones
@constraint(m, caprec[j = 1:rec], sum(X[i]*c_rec_prod[j,i] for i in 1:prod) <= disp_rec[j])

#print(m)

status = optimize!(m)

println("Utilidad total  : ", JuMP.objective_value(m))

for i in 1:prod
    println(X[i],  "            : ", round(JuMP.value(X[i]), digits=2))
end

# AQUI EMPIEZA EL ANÁLISIS DE SENSIBILIDAD

# IMPRESIÓN DE COSTOS REDUCIDOS
println("\nCOSTOS REDUCIDOS")
for i in 1:prod
    println("X[",i,"]:  ", round(reduced_cost(X[i]), digits=2))
end

# IMPRESIÓN DE PRECIOS SOMBRA
println("\nPRECIOS SOMBRA")
for j in 1:rec
    println(recursos[j], "   ", round(shadow_price(caprec[j]), digits=2))
end

# GENERACIÓN DEL REPORTE DE SENSIBILIDAD
sens_rep = lp_sensitivity_report(m)

# RANGOS DE VALIDEZ DE LOS COEF-OBJETIVO
rango_vars = zeros(prod,2)

for i in 1:prod
    rango_vars[i,1], rango_vars[i,2] = sens_rep[X[i]]
end

println("\nRangos de los coef en la función objetivo")
println("en los que la solución óptima no cambia\n")
println("             Min       Max")
println("           -------   -------")

for i in 1:prod
    minp = ut_prod[i] + round(rango_vars[i,1], digits=2)
    maxp = ut_prod[i] + round(rango_vars[i,2], digits=2)
    println("Producto ",i,"  ", minp, "     ", maxp)
end

# RANGOS DE VALIDEZ DE L0S PRECIOS SOMBRA
rango_rec = zeros(rec,2)

for j in 1:rec
    rango_rec[j,1], rango_rec[j,2] = sens_rep[caprec[j]]
end

println("\nRangos de los valores del lado derecho")
println("en los que son válidos los precios sombra\n")
println("           Min        Max")
println("          -------   -------")

for j in 1:rec
    minr = disp_rec[j] + round(rango_rec[j,1], digits=2)
    maxr = disp_rec[j] + round(rango_rec[j,2], digits=2)
    println(recursos[j], "  ", minr, "     ", maxr)
end

# HOLGURAS (Recursos sobrantes en las restricciones)

slacks = disp_rec .- value.(caprec)
println("\nLas holguras de los recursos son:")

for i in 1:length(slacks)
    println(recursos[i], ": ", slacks[i])
end

final = time()

println("\nEl tiempo de ejecución fue de ", (final - inicio), " segundos")

# El tiempo de ejecución fue de  segundos
