# Modelo TSP25 de Juan Pablo Orejuela

# Importamos los módulos necesarios
import time
import numpy as np
from pulp import *

# Función para calcular las distancias
# entre los nodos
def eucdist(coords):
    """Recibe un vector de numpy
       con las coordenadas de 
       los nodos. Regresa una 
       matriz de numpy con
       las distancias.
    """
    nodos = len(coords) 
    distancias = np.zeros((nodos,nodos))
    for i in range(nodos):
        for j in range(i+1, nodos):
            dist = np.sqrt((coords[i][1] - coords[j][1])**2 + (coords[i][2] - coords[j][2])**2)
            distancias[i][j] = dist
            distancias[j][i] = dist

    return distancias

# INICIA PROGRAMA PRINCIPAL
def main():
    start = time.time()
    # Leemos los datos de las coordenadas    
    coords = np.loadtxt("DJIB38-PuLP.txt")
    
    nodos = range(len(coords))
    st = range(1,len(coords))
    
    # Creamos la matriz de distancias entre todos los nodos
    distancias = eucdist(coords)
    
    # Creamos el modelo
    model = LpProblem(name="TSP25", sense=LpMinimize)
    
    # Creamos las variables binarias
    X = LpVariable.dicts("X", (nodos, nodos),0,1,LpInteger)
    
    # Creamos las variables para los subtours
    U = LpVariable.dicts("U", st, 0, None, LpContinuous)
    
    # Creamos la función objetivo
    model += lpSum(X[i][j] * distancias[i][j] for i in nodos for j in nodos if i != j)
    
    # Agregamos las restricciones
    # entrada_nodos
    for j in nodos:
        model += lpSum([X[i][j] for i in nodos if i!= j]) == 1, "EN " + str(j)
    # salida_nodos
    for i in nodos:
        model += lpSum([X[i][j] for j in nodos if j!= i]) == 1, "SN " + str(i)
    # subtours
    for i in nodos:
        for j in nodos:
            if (i != j) and (i>0) and (j>0):
                model += U[i]-U[j] + (len(coords)*X[i][j]) <= len(coords) - 1

    # Se resuelve el problema usando el solver por defecto (CBC)
    status = model.solve()

    # Se imprime el status del problema
    print("Status:", LpStatus[model.status])

    # Se imprime la función objetivo  
    print("Distancia Total = ", value(model.objective))

    # Se imprimen las variables
    for v in model.variables():
        if v.name[0] == 'X' and v.varValue != 0:
            print(v.name, "=", v.varValue)
    
    end = time.time()

    #Termina la medición de tiempo
    print("El tiempo de ejecución fue de ", (end - start), "segundos")

    # El tiempo de ejecución fue de 3.4899184703826904  segundos

if __name__ == "__main__":
    main()
